/* wagon-ESP-OSC
 *  un wagon radio-guidé
 *  plateforme ESP8266
 *  commande moteur Olimex BB-L298
 *  protocole réseau OSC
 *  SNHack gepeto@du-libre.org
 *  Codes sous licences GPLV3
 */

/* projet LOCO-DOM
 *  IP du réseau 192.168.0.x/24
 *  les loco sont de x= 81 a 91
 *  ports 9081 a 9091
 * Connexions
 * Olimex ESP8266-EVB |con3<=>BBL-298
 * |1 | 1| +5 |
 * |2 | 8| GND |
 * Olimex UEXT <=> BB-298
 * | 1  | 3.3V   |     | nc  |
 * | 2  | GND    |     | console debug GND |
 * | 3  | gpio 1 | TXD | console debug rx |
 * | 4  | gpio 3 | RXD | console debug tx |
 * | 5  | gpio 4 |   6 | nc |
 * | 6  | gpio 2 |   7 | nc |
 * | 7  | gpio 12 |  4 | in 2 |
 * | 8  | gpio 13 |  3 | in 1 |
 * | 9  | gpio 14 |  2 | enable A |
 * | 10 | gpio 15 |  5 | rfid tx |
 * + lecteur rfid
 */

// WIFI
#include <ESP8266WiFi.h>
#include <WiFiUdp.h>

//MDNSResponder mdns;
// OSC
#include <OSCBundle.h>
#include <OSCMessage.h>
#include <OSCBoards.h>
//port numbers for OSC
const unsigned int inPort = 9081;
WiFiUDP udp;
// serial
#include <EspSoftSerialRx.h>
EspSoftSerialRx rfid;
const char* ssid     = "LOCOMO";
const char* password = "0123456789";
const char nomhost[20]  = "LOCO-1";
//the IP
//IPAddress (192,168,IPC,IPD);
int IPC = 0;
int IPD = 81;
String nom = "LOCO-1";
const char* OSCVar = "/LOCO-1";
//destination IP pour la trace OSC
IPAddress outIp(192, 168, 0, 115);
const unsigned int outPort = 9999;

// connect motor controller pins to Arduino digital pins
// ESP moteur traction
int enA = 14;
int in1 = 12;
int in2 = 13;
float Vitesse = 0 ; // vitesse actuelle, <0 = arriere
float temps_boucle_total = 0 ; // en milli seconde

const int nbr_de_puces = 20;
int no_puce_actuelle;

// tableau des puces rfid 20x10
char *RFID[nbr_de_puces] = {
  "6D0070007964", // puce 0
  "0123456ab1",
  "0123456ab2",
  "0123456ab3",
  "0123456ab4",
  "0123456ab5",
  "0123456ab6",
  "0123456ab7",
  "0123456ab8",
  "0123456ab9",
  "1123456ab0",
  "1123456ab1",
  "1123456ab2",
  "1123456ab3",
  "1123456ab4",
  "1123456ab5",
  "1123456ab6",
  "1123456ab7",
  "1123456ab8",
  "1123456ab9",
};
static int32_t sequencenumber=0;
int commande=0; // no de la commande en cours

void setup()
{
  Serial.begin(115200);
  // set all the motor control pins to outputs
  pinMode(enA, OUTPUT);
  pinMode(in1, OUTPUT);
  pinMode(in2, OUTPUT);
  // serial RFID
  rfid.begin(9600, 2); // sortie UEXT 10 =15
  //setup ethernet part
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  WiFi.config(IPAddress(192, 168, IPC, IPD), IPAddress(192, 168, IPC, IPD), IPAddress(255, 255, 255, 0));
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println(ssid);
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  Serial.println("Starting UDP");
  udp.begin(inPort);
  Serial.print("Local port: ");
  Serial.println(udp.localPort());
  /* // pas de mDNS ca pas de DHCP sur l'AP?
   * if (mdns.begin(nomhost, WiFi.softAPIP())) {
    Serial.println("MDNS responder started");
    Serial.print("You can now connect to http://");
    Serial.print(nomhost);
    Serial.println(".local");
    // Add service to MDNS-SD
    //MDNS.addService("http", "tcp", 80);
  } else {
    Serial.println("pb mdns");
  }
  */
  
}

int litrfid() {
  // mise en forme de la lecture rfid
  //retourne le no_puce
  
  //Serial.println("attente rfid");
  int i = 0;
  unsigned char val;
  char tagValue[12];
  for (i = 0; i <= 11; i++) {
    if (rfid.read(val)) {
      tagValue[i] = '\0'; // Null
      // on saute avec le start 0x02 et stop bit 0x03
      if ((val == 0x0D) || (val == 0x0A) || (val == 0x03) || (val == 0x02)) {
        break;
      }
      //      Serial.print(i);
      //      Serial.print(" ");
      //      Serial.print((char)val);
      //      Serial.print(" ");
      //    Serial.println(val, HEX );
      tagValue[i] = (char)val;

      if ((i == 11) ) { // 11 digit read is complete
        tagValue[12] = '\0'; // Null
        //        Serial.print("Tag read: ");
                Serial.println(tagValue);
        // Search the tag database for this particular tag
        int tagId = CherchePuce( tagValue );
        // Only fire the strike plate if this tag was found in the database
        if ( tagId != 99 )
        {
          Serial.print("puce ");
          Serial.println(tagId);
          Message(tagId);
          return (tagId);
        } else {
          Serial.println("puce ?");
        }
      }
    }
  }
}

int CherchePuce( char tag[12] ) {
  for (int puce = 0; puce < nbr_de_puces; puce++) {
    // Check if the tag value matches this row in the tag database
    if (strcmp(tag, RFID[puce]) == 0)
      return (puce);
      break;
  }
  // Si pas trouvé on retourne 99
  return (99);
}


bool Message(int bla) { // envoie OSC /msg i position
  OSCBundle envoi;
  envoi.add("/LOCO-1").add((int32_t)bla);
  envoi.add("/sequencenumber").add(sequencenumber++);
  //msg.add((int32_t)analogRead(0));
  udp.beginPacket(outIp, outPort);
  envoi.send(udp); // envoi au WIFIudp
  udp.endPacket(); // mark the end of the OSC Packet
  envoi.empty(); // free space occupied by
  Serial.println("envoi");
  Serial.println(bla);
}

bool Aller(int no_puce, float vitesse) {
  // aller jusqu'a la puce no_puce a vitesse
  int tag;
  if (no_puce == no_puce_actuelle) {
    Message(no_puce);
    return true;
  }
  Vitesse_moteur(vitesse);
  while (tag = litrfid() != no_puce) { // roule tant que puce pas bonne
    // Message(tag);
    delay(100);
  }
  no_puce_actuelle = no_puce;
  return (true); // ? retourner le temps .. ///?///
}

void Vitesse_moteur(float moteur) {
  if (moteur > 1024) // filtre le max
    moteur = 1024;

  if (moteur < -1024)
    moteur = -1024;

  if (Vitesse > moteur) {
    for (float i = Vitesse; i == moteur; i--) {
      if ( i >= 0 ) {
        digitalWrite(in1, LOW);
        digitalWrite(in2, HIGH);
        analogWrite(enA, int(i));
      } else {
        digitalWrite(in1, HIGH);
        digitalWrite(in2, LOW);
        analogWrite(enA, int(-i));
      }
      delay(20); // rampe
    }

  } else {
    for (float i = Vitesse; i == moteur; i++) {
      if ( i >= 0 ) {
        digitalWrite(in1, LOW);
        digitalWrite(in2, HIGH);
        analogWrite(enA, int(i));
      } else {
        digitalWrite(in1, HIGH);
        digitalWrite(in2, LOW);
        analogWrite(enA, int(-i));
      }
      delay(20); // rampe
    }
  }
  Vitesse = moteur;
}
// COMMANDES
void Stop(OSCMessage &msg) {
  commande=0;
  //Serial.println(msg.match("/stop"));
  analogWrite(enA,0);

}
void MoteurControl(OSCMessage &msg) {
  // met le moteur à la vitesse reçue
  commande=1;
  //Serial.println(msg.match("/moteur"));
  float moteur_lu = msg.getFloat(0);
  Vitesse_moteur(moteur_lu);
}
void Initialisation(OSCMessage &msg) {       // A FINIR
  // la loco fait un tour complet pour connaitre le temps nécessaire
  // et vérifier la lecture des puces RFID , voir faire l'init du tableau?
  // elle ne connait au départ que la puce 0, position 0
  commande=2;
  //moteur 0
  Vitesse_moteur(0);
  //aller sur puce 0, init temps
  Aller(0, 800);
  temps_boucle_total = 0; 
  //rouler a fond,
  //  lire les puces, init du tableau, envoi /POS if(f) no_puce temps_en_ms (vitesse)
  //  des que puce 0 atteinte stocker temps_boucle_total nbre_de_puce
  // on positionnera les puces de facon regulière
}
void Rendezvous(OSCMessage &msg, int addrOffset) {       // A FINIR
  // /goto ii puce temps : se rendre a la puce xx en tant de temps
  commande=3;

}
void loop()
{
  // Lire la comande et executer le debut
  
  OSCBundle msgIN;
  //OSCMessage msgIN;
  int size;
  if ( (size = udp.parsePacket()) > 0)
  {
    while (size--)
      msgIN.fill(udp.read());

    if (!msgIN.hasError()) {
      msgIN.dispatch("/stop", Stop);            // stop le moteur immediatement commande=0
      msgIN.dispatch("/moteur", MoteurControl);  // met le moteur à la vitesse lue  commande=1
      //msgIN.route("/init", Initialisation); // fait un tour pour reperer les puces (puce0 fixe) commande=2
      //msgIN.route("/goto", Rendezvous);           // va à la puce dans le temps donné commande=3
      //msgIN.dispatch("/oueston", Oueston);            // renvoi la derniere puce lue et la vitesse commande=4

    }
    udp.flush();
  }
  // Lire une puce et continuer une commande en cours
  
  litrfid();
  delay(100);
}
